@@ -201,6 +201,9 @@ struct Config {
   // show_stat, if true, displays the connection statistics when the
   // connection is closed.
   bool show_stat;
+  // gso_burst is the number of packets to aggregate in GSO.  0 means
+  // it is not limited by the configuration.
+  size_t gso_burst;
 };
 
 class ClientBase {
