@@ -195,8 +195,8 @@ private:
       std::span<const uint8_t> data;
       size_t gso_size;
     } blocked;
-    std::unique_ptr<uint8_t[]> data;
   } tx_;
+  std::array<uint8_t, 64_k> txbuf_;
 };
 
 class Server {
