@@ -29,24 +29,28 @@
 #  include <config.h>
 #endif // defined(HAVE_CONFIG_H)
 
-#if defined(ENABLE_EXAMPLE_QUICTLS) && defined(WITH_EXAMPLE_QUICTLS)
+#ifdef WITH_EXAMPLE_QUICTLS
 #  include "tls_client_context_quictls.h"
-#endif // defined(ENABLE_EXAMPLE_QUICTLS) && defined(WITH_EXAMPLE_QUICTLS)
+#endif // defined(WITH_EXAMPLE_QUICTLS)
 
-#if defined(ENABLE_EXAMPLE_GNUTLS) && defined(WITH_EXAMPLE_GNUTLS)
+#ifdef WITH_EXAMPLE_GNUTLS
 #  include "tls_client_context_gnutls.h"
-#endif // defined(ENABLE_EXAMPLE_GNUTLS) && defined(WITH_EXAMPLE_GNUTLS)
+#endif // defined(WITH_EXAMPLE_GNUTLS)
 
-#if defined(ENABLE_EXAMPLE_BORINGSSL) && defined(WITH_EXAMPLE_BORINGSSL)
+#ifdef WITH_EXAMPLE_BORINGSSL
 #  include "tls_client_context_boringssl.h"
-#endif // defined(ENABLE_EXAMPLE_BORINGSSL) && defined(WITH_EXAMPLE_BORINGSSL)
+#endif // defined(WITH_EXAMPLE_BORINGSSL)
 
-#if defined(ENABLE_EXAMPLE_PICOTLS) && defined(WITH_EXAMPLE_PICOTLS)
+#ifdef WITH_EXAMPLE_PICOTLS
 #  include "tls_client_context_picotls.h"
-#endif // defined(ENABLE_EXAMPLE_PICOTLS) && defined(WITH_EXAMPLE_PICOTLS)
+#endif // defined(WITH_EXAMPLE_PICOTLS)
 
-#if defined(ENABLE_EXAMPLE_WOLFSSL) && defined(WITH_EXAMPLE_WOLFSSL)
+#ifdef WITH_EXAMPLE_WOLFSSL
 #  include "tls_client_context_wolfssl.h"
-#endif // defined(ENABLE_EXAMPLE_WOLFSSL) && defined(WITH_EXAMPLE_WOLFSSL)
+#endif // defined(WITH_EXAMPLE_WOLFSSL)
+
+#ifdef WITH_EXAMPLE_OSSL
+#  include "tls_client_context_ossl.h"
+#endif // defined(WITH_EXAMPLE_OSSL)
 
 #endif // !defined(TLS_CLIENT_CONTEXT_H)
