@@ -80,18 +80,23 @@ inline nghttp3_nv make_nv_nn(const std::string_view &name,
                  NGHTTP3_NV_FLAG_NO_COPY_NAME | NGHTTP3_NV_FLAG_NO_COPY_VALUE);
 }
 
-inline constexpr auto hexdigits = []() {
-  constexpr char LOWER_XDIGITS[] = "0123456789abcdef";
+inline constexpr char LOWER_XDIGITS[] = "0123456789abcdef";
 
-  std::array<char, 512> tbl;
+template <std::weakly_incrementable O>
+requires(std::indirectly_writable<O, char>)
+constexpr O format_hex_uint8(uint8_t b, O result) {
+#ifdef __GNUC__
+#  pragma GCC diagnostic push
+#  pragma GCC diagnostic ignored "-Wsign-conversion"
+#endif // __GNUC__
+  *result++ = LOWER_XDIGITS[b >> 4];
+  *result++ = LOWER_XDIGITS[b & 0xf];
+#ifdef __GNUC__
+#  pragma GCC diagnostic pop
+#endif // __GNUC__
 
-  for (size_t i = 0; i < 256; ++i) {
-    tbl[i * 2] = LOWER_XDIGITS[static_cast<size_t>(i >> 4)];
-    tbl[i * 2 + 1] = LOWER_XDIGITS[static_cast<size_t>(i & 0xf)];
-  }
-
-  return tbl;
-}();
+  return result;
+}
 
 // format_hex converts a range [|first|, |last|) in hex format, and
 // stores the result in another range, beginning at |result|.  It
@@ -102,9 +107,7 @@ requires(std::indirectly_writable<O, char> &&
          sizeof(std::iter_value_t<I>) == sizeof(uint8_t))
 constexpr O format_hex(I first, I last, O result) {
   for (; first != last; ++first) {
-    result = std::ranges::copy_n(
-               hexdigits.data() + static_cast<uint8_t>(*first) * 2, 2, result)
-               .out;
+    result = format_hex_uint8(static_cast<uint8_t>(*first), result);
   }
 
   return result;
@@ -172,7 +175,7 @@ template <std::unsigned_integral T, std::weakly_incrementable O>
 requires(std::indirectly_writable<O, char>)
 constexpr O format_hex(T n, O result) {
   if constexpr (sizeof(n) == 1) {
-    return std::ranges::copy_n(hexdigits.data() + n * 2, 2, result).out;
+    return format_hex_uint8(n, result);
   }
 
   if constexpr (std::endian::native == std::endian::little) {
@@ -180,15 +183,14 @@ constexpr O format_hex(T n, O result) {
     auto p = end + sizeof(n);
 
     for (; p != end; --p) {
-      result =
-        std::ranges::copy_n(hexdigits.data() + *(p - 1) * 2, 2, result).out;
+      result = format_hex_uint8(*(p - 1), result);
     }
   } else {
     auto p = reinterpret_cast<uint8_t *>(&n);
     auto end = p + sizeof(n);
 
     for (; p != end; ++p) {
-      result = std::ranges::copy_n(hexdigits.data() + *p * 2, 2, result).out;
+      result = format_hex_uint8(*p, result);
     }
   }
 
@@ -293,7 +295,7 @@ std::string_view strccalgo(ngtcp2_cc_algo cc_algo);
 std::optional<std::unordered_map<std::string, std::string>>
 read_mime_types(const std::string_view &filename);
 
-inline constexpr auto count_digit_tbl = []() {
+inline constexpr auto count_digit_tbl = [] {
   std::array<uint64_t, std::numeric_limits<uint64_t>::digits10> tbl;
 
   uint64_t x = 1;
@@ -321,7 +323,7 @@ template <std::unsigned_integral T> constexpr size_t count_digit(T x) {
   return y + 1;
 }
 
-inline constexpr auto utos_digits = []() {
+inline constexpr auto utos_digits = [] {
   std::array<char, 200> a;
 
   for (size_t i = 0; i < 100; ++i) {
@@ -480,7 +482,7 @@ constexpr bool is_hex_string(R &&r) {
   return !(std::ranges::size(r) & 1) && std::ranges::all_of(r, is_hex_digit);
 }
 
-inline constexpr auto hex_to_uint_tbl = []() {
+inline constexpr auto hex_to_uint_tbl = [] {
   std::array<uint32_t, 256> tbl;
 
   std::ranges::fill(tbl, 256);
